@@ -1,5 +1,6 @@
 from copy import deepcopy
 from types import DictType, FileType, ListType, StringTypes
+from DateTime import DateTime
 
 from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.Expression import Expression
@@ -8,7 +9,7 @@ from Products.CMFCore.Expression import createExprContext
 from Products.Archetypes.utils import className
 from Products.Archetypes.utils import unique
 from Products.Archetypes.utils import capitalize
-from Products.generator.widget import macrowidget
+from Products.Archetypes.generator import macrowidget
 from Products.Archetypes.debug import log
 from Products.Archetypes.Registry import registerPropertyType
 from Products.Archetypes.Registry import registerWidget
@@ -326,6 +327,9 @@ class TextAreaWidget(TypesWidget):
         'format': 0,
         'append_only': False,
         'divider':"\n\n========================\n\n",
+        'timestamp': True,
+        'maxlength' : False,
+        'helper_js': ('widgets/js/textcount.js',),        
         })
 
     security = ClassSecurityInfo()
@@ -356,15 +360,21 @@ class TextAreaWidget(TypesWidget):
             kwargs['mimetype'] = text_format
 
         """ handle append_only  """
-        # SPANKY: It would be nice to add a datestamp too, if desired
-
         # Don't append if the existing data is empty or nothing was passed in
         if getattr(field.widget, 'append_only', None):
             if field.getEditAccessor(instance)():
                 if (value and not value.isspace()):
+                    
+                    divider = field.widget.divider
+                    
+                    # Add a datestamp along with divider if desired.
+                    if getattr(field.widget, 'timestamp', None):
+
+                        divider = "\n\n" + str(DateTime()) + divider
+                        
                     # using default_output_type caused a recursive transformation
                     # that sucked, thus mimetype= here to keep it in line
-                    value = value + field.widget.divider + \
+                    value = value + divider + \
                             field.getEditAccessor(instance)()
                 else:
                     # keep historical entries
@@ -394,7 +404,6 @@ class CalendarWidget(TypesWidget):
     _properties.update({
         'macro' : "widgets/calendar",
         'format' : '', # time.strftime string
-        # the following five vars aren't supported by the plone templates yet
         'show_hm' : True, 
         'show_ymd' : True, # False not supported by the plone templates yet
         'starting_year' : 1999, # not supported by the plone templates yet
@@ -501,9 +510,9 @@ class FileWidget(TypesWidget):
 
         if fileobj is empty_marker: return empty_marker
 
-        filename = getattr(fileobj, 'filename', '') or \
-                   (isinstance(fileobj, FileType) and \
-                    getattr(fileobj, 'name', ''))
+        filename = getattr(fileobj, 'filename', '')
+        if not filename:
+            filename = getattr(fileobj, 'name', '')
 
         if filename:
             value = fileobj
@@ -582,6 +591,8 @@ class IdWidget(TypesWidget):
         'display_autogenerated' : True,
         # script used to determine if an ID is autogenerated
         'is_autogenerated' : 'isIDAutoGenerated',
+        # ignore global or by-member setting for visible ids?
+        'ignore_visible_ids': False,
         })
 
     security = ClassSecurityInfo()
@@ -605,7 +616,6 @@ class RequiredIdWidget(IdWidget):
 
     # XXX
     security.declarePublic('process_form')
-
     def process_form(self, instance, field, form, empty_marker=None):
         """Override IdWidget.process_form to require id."""
         return TypesWidget.process_form(self, instance, field, form, empty_marker)
@@ -891,6 +901,7 @@ registerPropertyType('rows', 'integer', TextAreaWidget)
 registerPropertyType('cols', 'integer', TextAreaWidget)
 registerPropertyType('append_only', 'boolean', TextAreaWidget)
 registerPropertyType('divider', 'string', TextAreaWidget)
+registerPropertyType('timestamp', 'boolean', TextAreaWidget)
 registerPropertyType('rows', 'integer', LinesWidget)
 registerPropertyType('cols', 'integer', LinesWidget)
 registerPropertyType('rows', 'integer', VisualWidget)
