@@ -5,8 +5,7 @@
 import os, sys
 if __name__ == '__main__':
     execfile(os.path.join(sys.path[0], 'framework.py'))
-
-from Testing import ZopeTestCase
+from textwrap import dedent
 
 from App.Common import rfc1123_date
 from DateTime import DateTime
@@ -16,14 +15,15 @@ from Interface.Verify import verifyObject
 
 from Products.CMFCore.utils import getToolByName
 
-from Products.PloneTestCase.PloneTestCase import PLONE21
+from Products.PloneTestCase.PloneTestCase import PLONE21, portal_owner, default_password
 
 from Products.ResourceRegistries.config import JSTOOLNAME
 from Products.ResourceRegistries.interfaces import IJSRegistry
-from Products.ResourceRegistries.tests import CSSRegistryTestCase
+from Products.ResourceRegistries.tests.RegistryTestCase import RegistryTestCase
+from Products.ResourceRegistries.tests.RegistryTestCase import FunctionalRegistryTestCase
+from Products.ResourceRegistries.tools.packer import test_suite as packer_tests
 
-
-class TestJSImplementation(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestJSImplementation(RegistryTestCase):
 
     def test_interfaces(self):
         tool = getattr(self.portal, JSTOOLNAME)
@@ -31,7 +31,7 @@ class TestJSImplementation(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless(verifyObject(IJSRegistry, tool))
 
 
-class TestJSTool(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestJSTool(RegistryTestCase):
 
     def testToolExists(self):
         self.failUnless(JSTOOLNAME in self.portal.objectIds())
@@ -43,7 +43,7 @@ class TestJSTool(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless(tool.manage_jsComposition())
 
 
-class TestJSSkin(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestJSSkin(RegistryTestCase):
 
     def testSkins(self):
         skins = self.portal.portal_skins.objectIds()
@@ -53,7 +53,7 @@ class TestJSSkin(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless(getattr(self.portal, 'renderAllTheScripts'))
 
 
-class testJSZMIMethods(CSSRegistryTestCase.CSSRegistryTestCase):
+class testJSZMIMethods(RegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -65,7 +65,7 @@ class testJSZMIMethods(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless(self.tool.getResources())
 
 
-class TestJSScriptRegistration(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestJSScriptRegistration(RegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -92,7 +92,7 @@ class TestJSScriptRegistration(CSSRegistryTestCase.CSSRegistryTestCase):
         self.assertEqual(len(self.tool.getResources()), 0)
 
 
-class TestJSScriptRenaming(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestJSScriptRenaming(RegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -124,7 +124,7 @@ class TestJSScriptRenaming(CSSRegistryTestCase.CSSRegistryTestCase):
         self.assertRaises(ValueError, self.tool.renameResource, 'spam', 'bacon')
 
 
-class TestJSToolSecurity(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestJSToolSecurity(RegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -143,7 +143,7 @@ class TestJSToolSecurity(CSSRegistryTestCase.CSSRegistryTestCase):
             self.fail()
 
 
-class TestJSToolExpression(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestJSToolExpression(RegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -167,7 +167,7 @@ class TestJSToolExpression(CSSRegistryTestCase.CSSRegistryTestCase):
                         'python:"eggs" in object.objectIds()', context))
 
 
-class TestJSScriptCooking(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestJSScriptCooking(RegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -202,12 +202,12 @@ class TestJSScriptCooking(CSSRegistryTestCase.CSSRegistryTestCase):
         self.tool.registerScript('spam spam spam', expression='string:spam')
         self.tool.registerScript('eggs')
         self.assertEqual(len(self.tool.getEvaluatedResources(self.folder)), 3)
-        ids = [item.getId() for item in self.tool.getEvaluatedResources(self.folder)]
-        self.failUnless('ham' in ids)
-        self.failUnless('eggs' in ids)
-        self.failIf('spam' in ids)
-        self.failIf('spam spam' in ids)
-        self.failIf('spam spam spam' in ids)
+        magic_ids = [item.getId() for item in self.tool.getEvaluatedResources(self.folder)]
+        self.failUnless('ham' in self.tool.concatenatedresources[magic_ids[0]])
+        self.failUnless('eggs' in self.tool.concatenatedresources[magic_ids[2]])
+        self.failUnless('spam' in self.tool.concatenatedresources[magic_ids[1]])
+        self.failUnless('spam spam' in self.tool.concatenatedresources[magic_ids[1]])
+        self.failUnless('spam spam spam' in self.tool.concatenatedresources[magic_ids[1]])
 
     def testGetEvaluatedScriptsWithExpression(self):
         self.tool.registerScript('ham')
@@ -235,9 +235,13 @@ class TestJSScriptCooking(CSSRegistryTestCase.CSSRegistryTestCase):
         self.tool.registerScript('ham', expression='string:ham')
         self.tool.registerScript('spam', expression='string:spam')
         evaluated = self.tool.getEvaluatedResources(self.folder)
-        evaluatedids = [item.getId() for item in evaluated]
-        self.failUnless(evaluatedids[0] == 'ham')
-        self.failUnless(evaluatedids[1] == 'spam')
+        magic_ids = [item.getId() for item in evaluated]
+        ids = []
+        for magic_id in magic_ids:
+            self.assertEqual(len(self.tool.concatenatedresources[magic_id]), 1)
+            ids.append(self.tool.concatenatedresources[magic_id][0])
+        self.failUnless(ids[0] == 'ham')
+        self.failUnless(ids[1] == 'spam')
 
     def testRenderingScriptLinks(self):
         self.tool.registerScript('ham')
@@ -245,9 +249,11 @@ class TestJSScriptCooking(CSSRegistryTestCase.CSSRegistryTestCase):
         self.tool.registerScript('spam', expression='string:spam')
         self.tool.registerScript('test_rr_1.css', inline='1')
         all = getattr(self.portal, 'renderAllTheScripts')()
+        evaluated = self.tool.getEvaluatedResources(self.folder)
+        magic_ids = [item.getId() for item in evaluated]
         self.failUnless('background-color' in all)
         self.failUnless('<script' in all)
-        self.failUnless('/spam' in all)
+        self.failUnless('/%s' %(magic_ids[1],) in all)
 
     def testReenderingConcatenatesInline(self):
         self.tool.registerScript('test_rr_1.css', inline='1')
@@ -264,7 +270,7 @@ class TestJSScriptCooking(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless('background-color' in renderedpage)
 
 
-class TestScriptMoving(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestScriptMoving(RegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -374,7 +380,7 @@ class TestScriptMoving(CSSRegistryTestCase.CSSRegistryTestCase):
         self.assertEqual(self.tool.getResourceIds(),
                          ('bacon', 'spam', 'eggs', 'ham'))
 
-class TestJSTraversal(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestJSTraversal(RegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -411,17 +417,15 @@ class TestJSTraversal(CSSRegistryTestCase.CSSRegistryTestCase):
         content = str(self.portal.restrictedTraverse('portal_javascripts/%s' % magicId))
 
 
-class TestPublishing(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestPublishing(FunctionalRegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
         self.tool.clearResources()
         self.toolpath = '/' + self.tool.absolute_url(1)
-        self.portalpath = '/' + getToolByName(self.portal, 'portal_url')(1)
+        self.folderpath = '/' + self.folder.absolute_url(1)
         self.tool.registerScript('test_rr_1.js')
-        self.setRoles(['Manager'])
-        self.portal.invokeFactory('Document', 'index_html')
-        self.setRoles(['Member'])
+        self.folder.invokeFactory('Document', 'index_html')
 
     def testPublishJSThroughTool(self):
         response = self.publish(self.toolpath + '/test_rr_1.js')
@@ -431,7 +435,7 @@ class TestPublishing(CSSRegistryTestCase.CSSRegistryTestCase):
     def testPublishNonMagicJSThroughTool(self):
         self.setRoles(['Manager'])
         body = """<dtml-var "'joined' + 'string'">"""
-        self.portal.addDTMLMethod('testmethod', file=body)
+        self.folder.addDTMLMethod('testmethod', file=body)
         self.tool.registerScript('testmethod')
         response = self.publish(self.toolpath + '/testmethod')
         self.assertEqual(response.getStatus(), 200)
@@ -439,14 +443,14 @@ class TestPublishing(CSSRegistryTestCase.CSSRegistryTestCase):
 
     def testPublishPageWithInlineJS(self):
         # This one fails from string/utf-8 concatenation
-        response = self.publish(self.portalpath)
+        response = self.publish(self.folderpath)
         self.assertEqual(response.getStatus(), 200)
         self.assertEqual(response.getHeader('Content-Type'),
                          'text/html;charset=utf-8')
         self.tool.clearResources()
         self.tool.registerScript('test_rr_1.js', inline=True)
         # Test that the main page retains its content-type
-        response = self.publish(self.portalpath)
+        response = self.publish(self.folderpath)
         self.assertEqual(response.getHeader('Content-Type'),
                          'text/html;charset=utf-8')
         self.assertEqual(response.getStatus(), 200)
@@ -456,15 +460,37 @@ class TestPublishing(CSSRegistryTestCase.CSSRegistryTestCase):
         # Test that the main page retains its content-type
         self.setRoles(['Manager'])
         body = """<dtml-call "REQUEST.RESPONSE.setHeader('Content-Type', 'text/javascript')">/*and some js comments too*/ """
-        self.portal.addDTMLMethod('testmethod', file=body)
+        self.folder.addDTMLMethod('testmethod', file=body)
         self.tool.registerScript('testmethod', inline=True)
-        response = self.publish(self.portalpath)
+        response = self.publish(self.folderpath)
         self.assertEqual(response.getHeader('Content-Type'),
                          'text/html;charset=utf-8')
         self.assertEqual(response.getStatus(), 200)
 
+class TestFivePublishing(FunctionalRegistryTestCase):
+    'Publishing with Five'
 
-class TestDebugMode(CSSRegistryTestCase.CSSRegistryTestCase):
+    def afterSetUp(self):
+        # Define some resource
+        from Products.Five.zcml import load_config
+        import Products.ResourceRegistries.tests
+        load_config('test.zcml', Products.ResourceRegistries.tests)
+        self.tool = getattr(self.portal, JSTOOLNAME)
+        self.tool.clearResources()
+        self.tool.registerScript('++resource++test_rr_1.js')
+        self.toolpath = '/' + self.tool.absolute_url(1)
+        self.portalpath = '/' + getToolByName(self.portal, 'portal_url')(1)
+        self.setRoles(['Manager'])
+        self.portal.invokeFactory('Document', 'index_html')
+        self.setRoles(['Member'])
+
+    def testPublishFiveResource(self):
+        response = self.publish(self.toolpath + '/++resource++test_rr_1.js')
+        self.assertEqual(response.getStatus(), 200)
+        self.assertEqual(response.getHeader('Content-Type')[:24], 'application/x-javascript')
+        self.assertEqual("window.alert('running')" in response.getBody(), True)
+
+class TestDebugMode(FunctionalRegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -501,54 +527,7 @@ class TestDebugMode(CSSRegistryTestCase.CSSRegistryTestCase):
         self.assertEqual(response.getHeader('Cache-Control'), 'max-age=0')
 
 
-class TestJSDefaults(CSSRegistryTestCase.CSSRegistryTestCase):
-
-    # These do not run for plone 2.1 +
-
-    def afterSetUp(self):
-        self.tool = getattr(self.portal, JSTOOLNAME)
-
-    def testClearingScripts(self):
-        self.failUnless(self.tool.getResources())
-        self.tool.clearResources()
-        self.failIf(self.tool.getResources())
-
-    def testDefaultsInstall(self):
-        scriptids = self.tool.getResourceIds()
-        self.failUnless('plone_menu.js' in scriptids)
-        self.failUnless('plone_javascript_variables.js' in scriptids)
-        self.failUnless('test_rr_1.js' in scriptids)
-
-    def testTraverseToConcatenatedDefaults(self):
-        scripts = self.tool.getEvaluatedResources(self.portal)
-        for s in scripts:
-            try:
-                magic = s.getId()
-                self.portal.restrictedTraverse('portal_javascripts/%s' % magic)
-            except KeyError:
-                self.fail()
-
-    def testUserConditionOnMenuScript(self):
-        scripts1 = self.tool.getEvaluatedResources(self.portal)
-        self.logout()
-        scripts2 = self.tool.getEvaluatedResources(self.portal)
-        self.failUnless(len(scripts1) > len(scripts2))
-
-    def testCallingOfConcatenatedScripts(self):
-        stylesheets = self.tool.getEvaluatedResources(self.portal)
-        for s in stylesheets:
-            if 'ploneScripts' in s.getId():
-                output = self.portal.restrictedTraverse(
-                             'portal_javascripts/%s' % s.getId())
-                break
-        if not output:
-            self.fail()
-        o = str(output)[:]
-        self.failIf('&lt;dtml-call' in o)
-        self.failIf('&amp;dtml' in o)
-        self.failUnless('portal_url' in o)
-
-class TestZODBTraversal(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestZODBTraversal(RegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -623,7 +602,7 @@ class TestZODBTraversal(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless('purple' in content)
         self.failIf('pink' in content)
 
-class TestResourcePermissions(CSSRegistryTestCase.CSSRegistryTestCase):
+class TestResourcePermissions(FunctionalRegistryTestCase):
 
     def afterSetUp(self):
         self.tool = getattr(self.portal, JSTOOLNAME)
@@ -671,13 +650,19 @@ class TestResourcePermissions(CSSRegistryTestCase.CSSRegistryTestCase):
     def testRaiseUnauthorizedOnPublish(self):
         response = self.publish(self.toolpath + '/testroot.js')
         #Will be 302 if CookieCrumbler is enabled
-        self.failUnless(response.getStatus() in [302, 403])
+        self.failUnless(response.getStatus() in [302, 403, 401])
 
     def testRemovedFromResources(self):
+        # This test assumes that content is not merged or cached
+        self.tool.unregisterResource('test_rr_1.js')
+        self.tool.registerResource('test_rr_1.js', cookable=False, cacheable=False)
         scripts = self.tool.getEvaluatedResources(self.portal)
         ids = [item.getId() for item in scripts]
         self.failIf('testroot.js' in ids)
         self.failUnless('test_rr_1.js' in ids)
+        # Return resources to normal (not sure if this is needed)
+        self.tool.unregisterResource('test_rr_1.js')
+        self.tool.registerScript('test_rr_1.js')
 
     def testRemovedFromMergedResources(self):
         self.tool.unregisterResource('testroot.js')
@@ -708,11 +693,14 @@ class TestResourcePermissions(CSSRegistryTestCase.CSSRegistryTestCase):
         except Unauthorized:
             self.fail()
 
-class TestMergingDisabled(CSSRegistryTestCase.CSSRegistryTestCase):
+    def testAuthorizedOnPublish(self):
+        authstr = "%s:%s" % (portal_owner, default_password)
+        response = self.publish(self.toolpath + '/testroot.js', basic=authstr)
+        self.failUnlessEqual(response.getStatus(), 200)
+
+class TestMergingDisabled(RegistryTestCase):
 
     def afterSetUp(self):
-        self.req_resources = 3
-        self.req_cooked = 2
         self.tool = getattr(self.portal, JSTOOLNAME)
         self.tool.clearResources()
         self.tool.registerScript('testroot.js')
@@ -739,23 +727,19 @@ class TestMergingDisabled(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failIf(self.tool.getResources()[self.tool.getResourcePosition('simple2.js')].getCookable())
 
     def testNumberOfResources(self):
-        req_resources = 3
-        req_cooked = 2
-        self.assertEqual(len(self.tool.getResources()), req_resources)
-        self.assertEqual(len(self.tool.cookedresources), req_cooked)
-        self.assertEqual(len(self.tool.concatenatedresources), req_resources + (req_resources - req_cooked ))
+        self.assertEqual(len(self.tool.getResources()), 3)
+        self.assertEqual(len(self.tool.cookedresources), 2)
+        self.assertEqual(len(self.tool.concatenatedresources), 4)
         scripts = self.tool.getEvaluatedResources(self.portal)
-        self.assertEqual(len(scripts), req_cooked)
+        self.assertEqual(len(scripts), 2)
 
     def testCompositionWithLastUncooked(self):
-        req_resources = 3
-        req_cooked = 2
         self.tool.moveResourceToBottom('simple2.js')
-        self.assertEqual(len(self.tool.getResources()), req_resources)
-        self.assertEqual(len(self.tool.cookedresources), req_cooked)
-        self.assertEqual(len(self.tool.concatenatedresources), req_resources + (req_resources - req_cooked ))
+        self.assertEqual(len(self.tool.getResources()), 3)
+        self.assertEqual(len(self.tool.cookedresources), 2)
+        self.assertEqual(len(self.tool.concatenatedresources), 4)
         scripts = self.tool.getEvaluatedResources(self.portal)
-        self.assertEqual(len(scripts), req_cooked)
+        self.assertEqual(len(scripts), 2)
         magicId = None
         for script in scripts:
             id = script.getId()
@@ -770,14 +754,12 @@ class TestMergingDisabled(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless('blue' in content)
 
     def testCompositionWithFirstUncooked(self):
-        req_resources = 3
-        req_cooked = 2
         self.tool.moveResourceToTop('simple2.js')
-        self.assertEqual(len(self.tool.getResources()), req_resources)
-        self.assertEqual(len(self.tool.cookedresources), req_cooked)
-        self.assertEqual(len(self.tool.concatenatedresources), req_resources + (req_resources - req_cooked ))
+        self.assertEqual(len(self.tool.getResources()), 3)
+        self.assertEqual(len(self.tool.cookedresources), 2)
+        self.assertEqual(len(self.tool.concatenatedresources), 4)
         scripts = self.tool.getEvaluatedResources(self.portal)
-        self.assertEqual(len(scripts), req_cooked)
+        self.assertEqual(len(scripts), 2)
         magicId = None
         for script in scripts:
             id = script.getId()
@@ -792,15 +774,13 @@ class TestMergingDisabled(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless('blue' in content)
 
     def testCompositionWithMiddleUncooked(self):
-        req_resources = 3
-        req_cooked = 3
         self.tool.moveResourceToTop('simple2.js')
         self.tool.moveResourceDown('simple2.js')
-        self.assertEqual(len(self.tool.getResources()), req_resources)
-        self.assertEqual(len(self.tool.cookedresources), req_cooked)
-        self.assertEqual(len(self.tool.concatenatedresources), req_resources + (req_resources - req_cooked ))
+        self.assertEqual(len(self.tool.getResources()), 3)
+        self.assertEqual(len(self.tool.cookedresources), 3)
+        self.assertEqual(len(self.tool.concatenatedresources), 5)
         scripts = self.tool.getEvaluatedResources(self.portal)
-        self.assertEqual(len(scripts), req_cooked)
+        self.assertEqual(len(scripts), 3)
         content = str(self.portal.restrictedTraverse('portal_javascripts/simple2.js'))
         self.failUnless('blue' in content)
         content = str(self.portal.restrictedTraverse('portal_javascripts/test_rr_1.js'))
@@ -809,8 +789,6 @@ class TestMergingDisabled(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless('green' in content)
 
     def testLargerCompositionWithMiddleUncooked(self):
-        req_cooked = 3
-        req_resources = 5
         self.setRoles(['Manager'])
         self.portal.invokeFactory('File',
                                    id='testpurple.js',
@@ -828,11 +806,11 @@ class TestMergingDisabled(CSSRegistryTestCase.CSSRegistryTestCase):
         self.tool.moveResourceToTop('simple2.js')
         self.tool.moveResourceDown('simple2.js', 2)
         #Now have [[green,running],blue,[purple,pink]]
-        self.assertEqual(len(self.tool.getResources()), req_resources)
-        self.assertEqual(len(self.tool.cookedresources), req_cooked)
-        self.assertEqual(len(self.tool.concatenatedresources), req_resources + (req_resources - req_cooked ))
+        self.assertEqual(len(self.tool.getResources()), 5)
+        self.assertEqual(len(self.tool.cookedresources), 3)
+        self.assertEqual(len(self.tool.concatenatedresources), 7)
         scripts = self.tool.getEvaluatedResources(self.portal)
-        self.assertEqual(len(scripts), req_cooked)
+        self.assertEqual(len(scripts), 3)
         magicIds = []
         for script in scripts:
             id = script.getId()
@@ -852,6 +830,7 @@ class TestMergingDisabled(CSSRegistryTestCase.CSSRegistryTestCase):
         content = str(self.portal.restrictedTraverse('portal_javascripts/simple2.js'))
         self.failUnless('blue' in content)
 
+
 def test_suite():
     from unittest import TestSuite, makeSuite
     suite = TestSuite()
@@ -867,18 +846,12 @@ def test_suite():
     suite.addTest(makeSuite(TestScriptMoving))
     suite.addTest(makeSuite(TestJSTraversal))
     suite.addTest(makeSuite(TestPublishing))
+    suite.addTest(makeSuite(TestFivePublishing))
     suite.addTest(makeSuite(TestDebugMode))
     suite.addTest(makeSuite(TestZODBTraversal))
     suite.addTest(makeSuite(TestMergingDisabled))
     suite.addTest(makeSuite(TestResourcePermissions))
 
-    if not PLONE21:
-        # We must not test for the defaults in Plone 2.1 because they are
-        # all different. Plone2.1 has tests in CMFPlone/tests for defaults
-        # and migrations
-        suite.addTest(makeSuite(TestJSDefaults))
-
-
     return suite
 
 if __name__ == '__main__':
